//************************************************************************
// Befehle fr das SPI Protokoll
// AVRPRPOM
//
// Bei USB geht es schneller wenn man Daten und Taktsignale in
// einem Puffer als Bitstream sammelt und dann zusammen sendet.
// FlushUSB() sendet dann den Bitstream.
//
// Holger Klabunde
// 06.08.2005
// Compiler Visual C++ V5.0
//************************************************************************
#include "stdafx.h"

#include "Global.h"
#include "ftd2xx.h"
#include "ftd2def.h"
#include "protos.h"
#include "port_io.h"
#include "spi_io.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SLOWWAIT  1  //Wartezeit zwischen den DataPolling_Zyklen
#define PREWAIT   4 //Wartezeit bevor DataPolling beginnt

//**************************************************
//Synchronisiert den SPI Bus im AVR
//bzw. schaltet den Programmiermodus frei
//**************************************************
int CAVRPromDlg::SPISync()
{
 int loops;
 unsigned char syncspi;

 switch(device) //Check fr mich ob Baustein eingetragen ist
  {
   case S1200: case S2313: case S2323: case S2343:
   case S4414: case S8515: case S2333: case S4433:
   case S4434: case S8535:
   case TINY12: case TINY15: case TINY13: case TINY26:
//   case Tiny22: //Geht hier nicht wegen S2343, macht aber nix
   case TINY2313:
   case TINY25: case TINY45: case TINY85:
   case MEGA103: case MEGA161:  case MEGA163:  case MEGA603:
   case MEGA8:   case MEGA16:   case MEGA323:  case MEGA64:
   case MEGA128: case MEGA162:  case MEGA8515: case MEGA8535:
   case MEGA165: case MEGA169: case MEGA32:
   case MEGA48:  case MEGA88:   case MEGA168:
   case MEGA325: case MEGA3250: case MEGA645: case MEGA6450: //Auch fr 329,3290,649,6490
   case MEGA640: case MEGA1280: case MEGA1281:
   case MEGA644: case MEGA2560: case MEGA2561:
   case AT90CAN128:
   break;
   default:
      sprintf(zeile,"Devicecode 0x%04X fehlt in SPISync",device);
      Msg(zeile);
      return NO_BURN;
    break;
  }

//Sonderbehandlung fr AT90S1200
//untersttzt kein SPI-Sync !!
 if(device==S1200)
  {
   SPIWrite(SPI_ENABLE_PROG1); //SPI Mode aktivieren Byte1
   SPIWrite(SPI_ENABLE_PROG2); //SPI Mode aktivieren Byte2
   SPIWrite(DONTCARE);  //SPI Mode aktivieren Byte3
   SPIWrite(DONTCARE);  //SPI Mode aktivieren Byte4
   FlushUSB();
   Wait1ms(PROGENABLE);
   return NO_ERRORS;
  }

 loops=0;

//##################
//SPI Sync mit SCK
 if(   device==S2313   || device==S4414   || device==S8515
    || device==TINY22  || device==S2333   || device==S4433 || device==S2323
	|| device==S2343   || device==S4434   || device==S8535
    || device==MEGA103 || device==MEGA603 || device==MEGA323
    || device==TINY12  || device==TINY15
   )
  {
   do
   {
    SPIWrite(SPI_ENABLE_PROG1); //SPI Mode aktivieren Byte1
    SPIWrite(SPI_ENABLE_PROG2); //SPI Mode aktivieren Byte2
    syncspi=SPIRead();		   //SPI Mode aktivieren Byte3 !!!
    SPIWrite(DONTCARE);  //SPI Mode aktivieren Byte4
//    FlushUSB();

    if(syncspi!=0x53)
     {
	  SCLK_ON(); SCLK_OFF(); //Einmal takten
//Mit RESET_ON/OFF wird man hier bei AVR's nix
	 }

    loops++;

//   }while(syncspi!=0x53 && loops<32);
   }while(syncspi!=0x53 && loops<64); //loops auf 64 erhht ! 32 ist zu klein
                                      //Sonst immer wieder Fehlermeldung

   Wait1ms(PROGENABLE);

   if(loops==64)
    {
	 Msg("SPISync fehlgeschlagen !");
	 return NO_BURN; //Fehler nach 32 Versuchen
	}
   return NO_ERRORS;
  }//if(program_mode==SPI1 || program_mode==SPI2)

//##################
//SPI Sync mit Reset
 if(   device==MEGA161 || device==MEGA163  || device==MEGA8
    || device==MEGA16  || device==MEGA64   || device==MEGA128
    || device==MEGA162 || device==MEGA8515 || device==MEGA8535
    || device==MEGA165 || device==MEGA169 || device==MEGA32   
    || device==MEGA48  || device==MEGA88   || device==MEGA168
    || device==TINY13  || device==TINY26   || device==TINY2313  
    || device==TINY25  || device==TINY45   || device==TINY85
    || device==MEGA325 || device==MEGA3250 || device==MEGA645 || device==MEGA6450 //Auch fr MEGA329,3290,649,6490
    || device==MEGA640 || device==MEGA1280 || device==MEGA1281
    || device==MEGA644 || device==MEGA2560 || device==MEGA2561
    || device==AT90CAN128
   )
  {
    do
	{
     //Alle vier Bytes des Kommandos immer senden
     SPIWrite(SPI_ENABLE_PROG1); //SPI Mode aktivieren Byte1
     SPIWrite(SPI_ENABLE_PROG2); //SPI Mode aktivieren Byte2
     syncspi=SPIRead();		   //SPI Mode aktivieren Byte3 !!!
     SPIWrite(DONTCARE);  //SPI Mode aktivieren Byte4
     FlushUSB();
  
     if(syncspi!=0x53) //stimmt sync !
	  {
	   //Wenn nicht RESET und nochmal versuchen
       //Dies nicht in V2.03
       //Wait1ms(RESETTIME);
	   RESET_ON(); //Schaltung kurz laufen lassen !!
//*****
//ATMega32 macht Probleme bei 16MHz wenn man ab hier
//Wartezyklen einfgt
//*****

//Quick Bugfix fr das ATMEGA32 16MHz Problem
       if(device==MEGA32)
        {
         RESET_OFF();
        }
       else
        {
         //Dies in V2.03
         Wait1ms(RESETTIME);
         RESET_OFF();
         //Dies in V2.03
         Wait1ms(RESETTIME);
        }
	  }

	 loops++;
//    }while(syncspi!=0x53 && loops<10);
    }while(syncspi!=0x53 && loops<64);

//   if(loops==10)
   if(loops==64)
    {
	 Msg("SPISync fehlgeschlagen !");
	 return NO_BURN;
	}
   Wait1ms(PROGENABLE);
  }

 return NO_ERRORS;
}

//**************************************************
//Ein Wort in den Pagebuffer bei ATMEGA schreiben
//**************************************************
void CAVRPromDlg::SPILoadPage(unsigned long adr,unsigned int dat)
{
 unsigned char lowadr,low,high;
 unsigned long buf;

 buf=adr;

 buf&=(pagesize-1); //Hier unbenutzte Adressbits ausblenden
 lowadr=(unsigned char)(buf&0x00FF); //Nur unteres Adressbyte ntig

 high=(unsigned char)((dat&0xFF00)>>8); //Oberes Datenbyte
 low=(unsigned char)(dat&0x00FF);       //Unteres Datenbyte

 SPIWrite(SPI_LOAD_PAGE); //Commando senden LowByte schreiben
 SPIWrite(DONTCARE); //Dummy
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(low);  //Lowbyte Datenwort senden

 SPIWrite(SPI_LOAD_PAGE|0x08); //Commando senden HighByte schreiben
 SPIWrite(DONTCARE); //Dummy
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(high);  //Highbyte Datenwort senden
 FlushUSB();

}

//**************************************************
//Schreiben des Pagebuffer bei ATMEGA starten
//Oberen Adressteil der Page senden
//**************************************************
void CAVRPromDlg::SPIWritePage(unsigned long adr)
{
 unsigned char extadr,lowadr,highadr;
 unsigned long buf;

 buf=adr;
 extadr=(unsigned char)((adr&0xFF0000)>>16); //Extended Adressbyte
 highadr=(unsigned char)((adr&0xFF00)>>8); //Oberes Adressbyte

 buf&=~(pagesize-1); //Hier unbenutzte Adressbits ausblenden
 lowadr=(unsigned char)(buf&0x00FF); //unteres Adressbyte

 //Extended Adress setzen
 if(   device==MEGA640  || device==MEGA1280  || device==MEGA1281
    || device==MEGA644  || device==MEGA2560 || device==MEGA2561)
  {
   //Brauche ich das wirklich fr 640,1280,1281 ?
   SPIWrite(SPI_EXT_ADR_SET1);
   SPIWrite(SPI_EXT_ADR_SET2);
   SPIWrite(extadr);
   SPIWrite(DONTCARE); //Dummy
  }

 SPIWrite(SPI_WRITE_PAGE);    //Commando WritePage senden
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(DONTCARE);  //Dummy
 FlushUSB();

}

//**************************************************
//Ein Wort in den Programmspeicher des AVR schreiben
//**************************************************
void CAVRPromDlg::SPIWriteAVRWord(unsigned long adr,unsigned int dat)
{
 unsigned char lowadr,highadr,low,high,by;
 unsigned long counter;

 highadr=(unsigned char)((adr&0xFF00)>>8); //Oberes Adressbyte
 lowadr=(unsigned char)(adr&0x00FF);       //Unteres Adressbyte

 high=(unsigned char)((dat&0xFF00)>>8); //Oberes Datenbyte
 low=(unsigned char)(dat&0x00FF);       //Unteres Datenbyte

//Sonderbehandlung fr AT90S1200
 if(device==S1200)
  {
   SPIWrite(SPI_WRITE_PROGRAM); //Commando senden LowByte schreiben
   SPIWrite(highadr); //Highadr senden
   SPIWrite(lowadr);  //Lowadr senden
   SPIWrite(low);  //Lowbyte Datenwort senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_PROGRAM|0x08); //Commando senden HighByte schreiben
   SPIWrite(highadr); //Highadr senden
   SPIWrite(lowadr);  //Lowadr senden
   SPIWrite(high);  //Highbyte Datenwort senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
   return;
  }

 SPIWrite(SPI_WRITE_PROGRAM); //Commando senden LowByte schreiben
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(low);  //Lowbyte Datenwort senden
 FlushUSB();

 if(use_usb) Wait1ms(progtime); //Bei USB kein Datapolling machen ! Dauert zu lange

 if(use_lpt)
  {
   Wait1ms(PREWAIT); //Einen Teil der Programmierzeit abwarten

   //DataPolling
   counter=0;
   if(low==0xFF || low==0x7F) //Vorsichtshalber
    {
     Wait1ms(progtime); //Programmierzeit abwarten
    }
   else
    {
     do
      {
       SPIWrite(SPI_READ_PROGRAM); //Commando senden LowByte lesen
       SPIWrite(highadr); //Highadr senden
       SPIWrite(lowadr);  //Lowadr senden
       by=SPIRead();
       if(by!=low) Wait1ms(SLOWWAIT);
       counter+=SLOWWAIT;
      }while(counter<progtime && by!=low);
     }
  }// if(use_lpt)

 SPIWrite(SPI_WRITE_PROGRAM|0x08); //Commando senden HighByte schreiben
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(high);  //Highbyte Datenwort senden
 FlushUSB();

 if(use_usb) Wait1ms(progtime); //Bei USB kein Datapolling machen ! Dauert zu lange
 if(use_lpt)
  {
   Wait1ms(PREWAIT); //Einen Teil der Programmierzeit abwarten

   //DataPolling
   counter=0;
   if(high==0xFF || high==0x7F) //Vorsichtshalber
    {
     Wait1ms(progtime); //Programmierzeit abwarten
    }
   else
    {
     do
      {
        SPIWrite(SPI_READ_PROGRAM|0x08); //Commando senden HighByte lesen
        SPIWrite(highadr); //Highadr senden
        SPIWrite(lowadr);  //Lowadr senden
        by=SPIRead();
	    if(by!=high) Wait1ms(SLOWWAIT);
	    counter+=SLOWWAIT;
      }while(counter<progtime && by!=high);
    }

 }// if(use_lpt)

}

//***********************************************
//Ein Wort aus dem Programmspeicher des AVR lesen
//***********************************************
unsigned int CAVRPromDlg::SPIReadAVRWord(unsigned long adr)
{
 unsigned int buff;
 unsigned char lowadr,highadr,extadr,low,high;

 extadr=(unsigned char)((adr&0xFF0000)>>16); //Ext. Adressbyte
 highadr=(unsigned char)((adr&0xFF00)>>8); //Oberes Adressbyte
 lowadr=(unsigned char)(adr&0x00FF);       //Unteres Adressbyte

 //Extended Adress setzen
 if(   device==MEGA640  || device==MEGA1280  || device==MEGA1281
    || device==MEGA644  || device==MEGA2560 || device==MEGA2561)
  {
   //Brauche ich das wirklich fr 640,1280,1281 ?
   SPIWrite(SPI_EXT_ADR_SET1);
   SPIWrite(SPI_EXT_ADR_SET2);
   SPIWrite(extadr);
   SPIWrite(DONTCARE); //Dummy
  }

 SPIWrite(SPI_READ_PROGRAM); //Commando senden LowByte lesen
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 low=SPIRead();

 SPIWrite(SPI_READ_PROGRAM|0x08); //Commando senden HighByte lesen
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 high=SPIRead();

 buff=(unsigned int)(high<<8);
 buff+=low;

 return buff;
}

//***********************************************
//Ein Byte in den Datenspeicher des AVR schreiben
//***********************************************
void CAVRPromDlg::SPIWriteAVRByte(unsigned long adr,unsigned char dat)
{
 unsigned char lowadr,highadr;
 unsigned long counter;
 unsigned char by;

 highadr=(unsigned char)((adr&0xFF00)>>8); //Oberes Adressbyte
 lowadr=(unsigned char)(adr&0x00FF);       //Unteres Adressbyte

 SPIWrite(SPI_WRITE_EEPROM); //Commando senden Byte schreiben
 SPIWrite(highadr); //Highadr senden
 SPIWrite(lowadr);  //Lowadr senden
 SPIWrite(dat);     //Datenbyte senden
 FlushUSB();

 if(use_usb) Wait1ms(progtime); //Bei USB kein Datapolling machen ! Dauert zu lange
 if(use_lpt)
  {
   Wait1ms(PREWAIT); //Einen Teil der Programmierzeit abwarten

   //DataPolling
   counter=0;
   if(dat==0xFF || dat==0x7F) //Vorsichtshalber
    {
     Wait1ms(progtime); //Programmierzeit abwarten
    }
   else
    {
     do
      {
        SPIWrite(SPI_READ_EEPROM); //Commando senden Byte lesen
        SPIWrite(highadr); //Highadr senden
        SPIWrite(lowadr);  //Lowadr senden
        by=SPIRead();
	    if(by!=dat) Wait1ms(SLOWWAIT);
	    counter+=SLOWWAIT;
      }while(counter<progtime && by!=dat);
    }

 }// if(use_lpt)

}

//***********************************************
//Ein Wort aus dem Datenspeicher des AVR lesen
//***********************************************
unsigned char CAVRPromDlg::SPIReadAVRByte(unsigned long adr)
{
 unsigned char buff;
 unsigned char lowadr,highadr;

 highadr=(unsigned char)((adr&0xFF00)>>8); //Oberes Adressbyte
 lowadr=(unsigned char)(adr&0x00FF);       //Unteres Adressbyte

 SPIWrite(SPI_READ_EEPROM); //Commando senden Byte lesen
 SPIWrite(highadr);         //Highadr senden
 SPIWrite(lowadr);          //Lowadr senden
 buff=SPIRead();            //Daten lesen

 return buff;
}

//***********************************************
//Schreibt die LockBits in AVR, ATMEGA
//***********************************************
void CAVRPromDlg::SPIWriteLockBits()
{
 unsigned char by;

 switch(device) //Check fr mich ob Baustein eingetragen ist
  {
   case S1200: case S2313: case S2323: case S2343:
   case S4414: case S8515: case S2333: case S4433:
   case S4434: case S8535:
   case TINY12: case TINY15: case TINY13: case TINY26:
//   case Tiny22: //Geht hier nicht wegen S2343, macht aber nix
   case TINY2313:
   case TINY25: case TINY45: case TINY85:
   case MEGA103: case MEGA161: case MEGA163:  case MEGA603:
   case MEGA8:   case MEGA16:  case MEGA323:  case MEGA64:
   case MEGA128: case MEGA162: case MEGA8515: case MEGA8535:
   case MEGA165: case MEGA169: case MEGA32:
   case MEGA48:  case MEGA88:  case MEGA168:
   case MEGA325: case MEGA3250: case MEGA645: case MEGA6450: //Auch fr 329,3290,649,6490
   case MEGA640: case MEGA1280:   case MEGA1281:
   case MEGA644: case MEGA2560:   case MEGA2561:
   case AT90CAN128:
   break;
   default:
      sprintf(zeile,"Devicecode 0x%04X fehlt in SPIWriteLockBits",device);
      Msg(zeile);
   break;
  }

 if(   device==S1200  || device==S2313  || device==S4414 || device==S8515
    || device==S2333  || device==S4433  || device==S2323
	|| device==S2343  || device==S4434  || device==S8535 || device==MEGA103
    || device==TINY12 || device==TINY15 || device==TINY22
	|| device==MEGA603  
   )
  {
   by=lockbits<<1;				//Lock2,1 kommen auf D1,0 rein, mssen aber auf D2,1
   by|=0xF9;					//Alle Bits auer D2,1 auf 1 setzen !
   SPIWrite(SPI_WRITE_LOCK_A);	//Kommando senden
   SPIWrite(by);				//Zweiten Teil des Kommandos mit Lockbits senden
   SPIWrite(DONTCARE);			//Fllcodes senden
   SPIWrite(DONTCARE);
   FlushUSB();
  }

 if(   device==MEGA161  || device==MEGA163 || device==MEGA8 || device==MEGA165
    || device==MEGA16   || device==MEGA64  || device==MEGA323
    || device==MEGA128  || device==MEGA162 || device==MEGA8515
    || device==MEGA8535 || device==MEGA169 || device==MEGA32
    || device==MEGA48   || device==MEGA88  || device==MEGA168
    || device==TINY13   || device==TINY26  || device==TINY2313
    || device==TINY25   || device==TINY45   || device==TINY85
//Auch fr MEGA329,3290,649,6490
    || device==MEGA325  || device==MEGA3250 || device==MEGA645 || device==MEGA6450
    || device==MEGA640  || device==MEGA1280 || device==MEGA1281
    || device==MEGA644  || device==MEGA2560 || device==MEGA2561
    || device==AT90CAN128
   )
  {
   by=lockbits;
   by|=0xC0;					//Alle unbenutzten Bits auf 1 setzen !
   if(device==TINY13 || device==TINY26 || device==TINY2313) by|=0xFC;
   if(device==TINY25 || device==TINY45 || device==TINY85) by|=0xFC;
   if(device==MEGA48) by|=0xFC;

   SPIWrite(SPI_WRITE_LOCK_A); //Kommando senden
   SPIWrite(0xFF);             //Bit D7..5 mssen 1 sein alle anderen x
   SPIWrite(DONTCARE);         //Fllcode
   SPIWrite(by);               //Lockbits senden
   FlushUSB();
  }

 Wait1ms(progtime); //Programmierzeit abwarten
}

//***********************************************
//Liest Lockbits bei ATMEGA
//***********************************************
unsigned char CAVRPromDlg::SPIReadLockBits()
{
 unsigned char by;

 SPIWrite(SPI_READ_LOCK);     //Kommando senden
 SPIWrite(DONTCARE);          
 SPIWrite(DONTCARE);  
 by=SPIRead();

 return by;
}

//***********************************************
//Schreibt die FuseBits
//***********************************************
void CAVRPromDlg::SPIWriteFuseBits()
{
 unsigned char by;

 by=0; //Besser is das

 switch(device) //Check fr mich ob Baustein eingetragen ist
  {
   case S1200:  case S2313: case S2323: case S2343:
   case S4414:  case S8515: case S2333: case S4433:
   case S4434:  case S8535:
   case TINY12: case TINY13: case TINY15: case TINY26:
//   case Tiny22: //Geht hier nicht wegen S2343, macht aber nix
   case TINY2313:
   case TINY25: case TINY45: case TINY85:
   case MEGA103:  case MEGA128:  case MEGA161:  case MEGA163: case MEGA603:
   case MEGA8:    case MEGA16:   case MEGA323: case MEGA162:  case MEGA165:
   case MEGA8515: case MEGA8535: case MEGA169: case MEGA32: case MEGA64:
   case MEGA48:    case MEGA88:   case MEGA168:
   case MEGA325: case MEGA3250: case MEGA645: case MEGA6450: //Auch fr 329,3290,649,6490
   case MEGA640: case MEGA1280:   case MEGA1281:
   case MEGA644: case MEGA2560:   case MEGA2561:
   case AT90CAN128:
   break;
   default:
      sprintf(zeile,"Devicecode 0x%04X fehlt in SPIWriteFuseBits",device);
      Msg(zeile);
   break;
  }

 if(   device==S1200  || device==S2313 || device==S4414 || device==S8515
    || device==TINY22 //wegen gleicher ID mit S2343
   )
  {
   return; //haben keine Fusebits bei SPI !
  }

 if(   device==S2333   || device==S4433
    || device==S2323   || device==S2343
	|| device==S4434   || device==S8535
	|| device==MEGA103 || device==MEGA603  
   )
  {
   by=0xA0;                      //D7..5 auf 101xxxxx
   by|=(fuselow&0x1F);           //Fusebits reinodern 
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(by);                 //Fusebits senden  
   SPIWrite(DONTCARE);          
   SPIWrite(DONTCARE);  
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==TINY12)
  {
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xBF);               //D7..5 auf 101xxxxx  
   SPIWrite(DONTCARE);          
   by=fuselow;                  //Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==TINY13)
  {
   //zwei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=0x00;                       
   by|=fuselow;                   //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0x00;                      
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==TINY15)
  {
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xBF);               //D7..5 auf 101xxxxx  
   SPIWrite(DONTCARE);          
   by=0x0C;                      //D3,2 auf 1
   by|=fuselow;                  //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==TINY2313)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;
   by|=0x01;                 //RSTDISBL verhindern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xFE;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==TINY26)
  {
   //zwei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=0x00;                       
   by|=fuselow;                   //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0xE0;                      //D7..5 auf 1
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

  if(device==TINY25 || device==TINY45 || device==TINY85)
   {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0x80;                      //RSTDISBL verhindern
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xFE;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
   }

 if(device==MEGA161)
  {
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xBF);               //D7..5 auf 101xxxxx  
   SPIWrite(DONTCARE);          
   by=0x80;                      //D7 auf 1
   by|=fuselow;                  //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA162)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=0x00;                      //D5,4 auf 1
   by|=fuselow;                   //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0x00;                      //D7..3 auf 1
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xE1;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA163)
  {
   //zwei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=0x30;                      //D5,4 auf 1
   by|=fuselow;                   //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0xF8;                      //D7..3 auf 1
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA165)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   SPIWrite(fusehigh);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF1;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA169)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=0x00;                      //D5,4 auf 1
   by|=fuselow;                   //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=0x00;                      //D7..3 auf 1
   by|=fusehigh;                 //Fusebits reinodern
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF0;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(   device==MEGA8    || device==MEGA16 || device==MEGA32
    || device==MEGA8515 || device==MEGA8535)
  {
   //zwei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=fuselow;                   //Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;                  //Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA323)
  {
   //zwei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=fuselow;                   //Fusebits
   by|=0x30;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;                  //Fusebits
   by|=0x30;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

//Auch fr MEGA329,3290,649,6490
 if(device==MEGA325 || device==MEGA3250 || device==MEGA645 || device==MEGA6450)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   SPIWrite(fusehigh);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF9;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(   device==MEGA640  || device==MEGA1280  || device==MEGA1281
    || device==MEGA644  || device==MEGA2560 || device==MEGA2561)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   SPIWrite(fusehigh);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF8;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA64 || device==MEGA128)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=fuselow;                   //Low Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;                  //High Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xFC;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA48)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=fuselow;                   //Low Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;                  //High Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xFE;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==MEGA88 || device==MEGA168)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   by=fuselow;                   //Low Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   by=fusehigh;                  //High Fusebits
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF8;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }

 if(device==AT90CAN128)
  {
   //drei FuseBytes !
   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA0);               //10100000 
   SPIWrite(DONTCARE);          
   SPIWrite(fuselow);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA8);               //10101000 
   SPIWrite(DONTCARE);          
   SPIWrite(fusehigh);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten

   SPIWrite(SPI_WRITE_FUSE);     //Kommando senden
   SPIWrite(0xA4);               //10100100 
   SPIWrite(DONTCARE);          
   by=fusex;                     //Extended Fusebits
   by|=0xF1;                     //unbenutzte Bits auf 1
   SPIWrite(by);                 //und senden
   FlushUSB();
   Wait1ms(progtime); //Programmierzeit abwarten
  }
}

//***********************************************
//Liest Fusebits
//***********************************************
unsigned char CAVRPromDlg::SPIReadFuseBits()
{
 unsigned char by;
 by=0;

 //Achtung bei AT2333,4433 !! Kommandocode falsch ?
 //Unterschiede bei ATMEGA !!!!
 return by;
}

//***********************************************
//Liest RC-Calibration bei ATMEGA163
//***********************************************
unsigned char CAVRPromDlg::SPIReadRCCal()
{
 unsigned char by;

 SPIWrite(0x38);     //Kommando senden
 SPIWrite(DONTCARE);
 SPIWrite(0x00);  
 by=SPIRead();

 return by;
}


//***********************************************
//Liest Fuses und Lockbits bei AVR's
//***********************************************
unsigned char CAVRPromDlg::SPIReadFuseLock()
{
 unsigned char by;

 SPIWrite(SPI_READ_LOCK_FUSE); //Kommando senden
 SPIWrite(DONTCARE); //Pseudo Kommandos senden, werden nicht benutzt
 SPIWrite(DONTCARE);  
 by=SPIRead();

 return by;
}

//***********************************************
//Liest eine Device-ID Byte von Adresse adr
//***********************************************
unsigned char CAVRPromDlg::SPIReadID(int adr)
{
 unsigned char by;

 SPIWrite(SPI_READ_SIGN_1); //Kommando senden
 if(device==TINY13) SPIWrite(SPI_READ_SIGN_3);
 else SPIWrite(SPI_READ_SIGN_2); //Kommando senden
 SPIWrite(adr);  //Adresse der ID senden
 by=SPIRead();

 return by;
}

//***********************************************
//Schreibt das RCENBit in den AVR
//***********************************************
void CAVRPromDlg::SPIWriteRcen(unsigned char rc)
{
 unsigned char by;

 by=SPI_WRITE_RCEN2; //Kommandocode mit RCEN gelscht
 if((rc&0x01)==0) by&=~0x01; //RCEN setzen, aktiv 0

 SPIWrite(SPI_WRITE_RCEN1); //Kommando senden
 SPIWrite(by);				//Zweiten Teil des Kommandos mit RCENBit senden
 SPIWrite(DONTCARE); //Pseudocodes weil nicht benutzt
 SPIWrite(DONTCARE);
 FlushUSB();

 Wait1ms(progtime); //Programmierzeit abwarten

}

//***********************************************
//Schreibt ein Byte in den SPI Baustein
//***********************************************
void CAVRPromDlg::SPIWrite(unsigned char by)
{
 unsigned char mask,i;

 mask=0x80;	//MSB first

 for(i=0; i<8; i++)
  {
   if((by&mask)>0) { SI_ON(); } //1 Bit
   else { SI_OFF(); }          //0 Bit

   SCLK_ON();  //Bit reintakten
   SCLK_OFF();
   mask=mask>>1;
  }

//Alternative Routine mit weniger Datenverkehr
//Bringt aber nichts. Also lieber wieder die oben nehmen.
/*
 //Das erste Datenbit vor SCLK_ON() senden
 if((by&mask)>0) { SI_ON(); } //1 Bit
 else { SI_OFF(); }          //0 Bit
 mask=mask>>1;

 for(i=0; i<8; i++)
  {
   SCLK_ON();  //Bit reintakten

   //Nchstes Datenbit schonmal vorbereiten, aber nicht senden
   if((by&mask)>0) { SID_ON(); } //1 Bit
   else { SID_OFF(); }          //0 Bit

   SCLK_OFF(); //nchstes Datenbit schon mit der fallenden Flanke ausgeben
   mask=mask>>1;
  }
*/
}

//***********************************************
//Liest ein Byte
//***********************************************
unsigned char CAVRPromDlg::SPIRead()
{
 unsigned char i,by,be;

 by=0;  //Bits kommen noch

// SO_ON(); //Pullup SO

//Achtung !! das erste Bit wurde mit der letzten negativen
//Flanke der Kommandosequenz rausgetaktet. Deshalb erst Bit abholen
//und dann CLK-Impuls beenden

 for(i=0; i<8; i++) //MSB first
  {
	SCLK_ON();
    FlushUSB();
	be=ReadByte(); //Laut Datenblatt in der Highphase lesen
    if(be>0) by|=(0x80>>i); //ein 1 Bit !!
	SCLK_OFF();
  }

// FlushUSB();
//Das letzte SCLK_OFF() wird bei USB nur in den Buffer
//bernommen und mit dem nchsten Befehl ausgegeben.

//Diese Funktion ist der Flaschenhals bei USB.
//Das zurcklesen der Bits kann nicht gepuffert werden
//weil SCLK bei jedem Bit takten mu. Beim senden sind
//dann auch nur max. zwei Bits im Puffer und mssen mit
//FlushUSB() geschrieben werden.
 return by;
}
